package com.example.tasks.viewModels

import android.content.Context
import android.util.Log
import androidx.compose.runtime.State
import androidx.compose.runtime.mutableStateOf
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.tasks.components.Constants
import com.example.tasks.components.SharedPreferenceHelper
import com.example.tasks.model.UserState
import io.github.jan.supabase.gotrue.OtpType
import io.github.jan.supabase.gotrue.auth
import io.github.jan.supabase.gotrue.providers.builtin.Email
import io.github.jan.supabase.gotrue.providers.builtin.OTP
import kotlinx.coroutines.launch


class AuthViewModel : ViewModel() {
    private val _userState = mutableStateOf<UserState>(UserState.Loading)
    val userState: State<UserState> = _userState
    var userID = ""
    fun onSignInEmailCode(context: Context, emailUser: String) {
        viewModelScope.launch {
            try {
                _userState.value = UserState.Loading
                 Constants.supabase.auth.signInWith(OTP) {
                    email = emailUser
                    createUser = false
                }
                saveToken(context)
                userID = Constants.supabase.auth.currentUserOrNull()!!.id
                _userState.value = UserState.Success("Successful sign in")

            } catch (e: Exception) {
                _userState.value = UserState.Error("Error: ${e.message}")

            }

        }
    }

    private fun saveToken(context: Context) {
        viewModelScope.launch {
            val accessToken = Constants.supabase.auth.currentAccessTokenOrNull()
            val sharedPref = SharedPreferenceHelper(context)
            sharedPref.saveStringData("accessToken", accessToken)
        }
    }

    private fun getToken(context: Context): String? {
        val sharedPref = SharedPreferenceHelper(context)
        return sharedPref.getStringData("accessToken")

    }

    fun onSignUpEmailCode(emailUser: String) {
        viewModelScope.launch {
            try {
                Constants.supabase.auth.signInWith(OTP) {
                    email = emailUser
                    createUser = true
                }

            } catch (e: Exception) {
                println(e.message.toString())

            }

        }
    }

    fun verifyOTP(emailUser: String, token: String) {
        viewModelScope.launch {
            try {
                _userState.value = UserState.Loading
                Constants.supabase.auth.verifyEmailOtp(
                    type = OtpType.Email.EMAIL,
                    email = emailUser,
                    token = token
                )
                _userState.value = UserState.Success("the correct code")
                Log.d("o", "gooood")

            } catch (e: Exception) {
                Log.d("o", "baaad")
                _userState.value = UserState.Error("Error: ${e.message}")

            }

        }
    }

    fun onSignInEmailPassword(emailUser: String, passwordUser: String) {
        viewModelScope.launch {
            try {
                val user = Constants.supabase.auth.signInWith(Email) {
                    email = emailUser
                    password = passwordUser
                }
                println(user.toString())
                println(Constants.supabase.auth.currentUserOrNull()!!.id)
                println("Success")
            } catch (e: Exception) {
                println("Error")
                println(e.message.toString())
            }
        }
    }

    fun onSignUpEmail(emailUser: String, passwordUser: String) {
        viewModelScope.launch {
            try {
                var user = Constants.supabase.auth.signUpWith(Email) {
                    email = emailUser
                    password = passwordUser
                }
                println(user.toString())
                println(Constants.supabase.auth.currentUserOrNull()!!.id)
                println("Success")
            } catch (e: Exception) {
                println("Error")
                println(e.message.toString())
            }

        }
    }


    fun isUserLoggedIn(context: Context){
        viewModelScope.launch {
            try {
                val token = getToken(context)
                if (token.isNullOrEmpty()){
                    _userState.value = UserState.Error("User is not logged in")
                }
                else
                {
                    Constants.supabase.auth.retrieveUser(token)
                    Constants.supabase.auth.refreshCurrentSession()
                    saveToken(context)
                    _userState.value = UserState.Success("User is logged in")
                }

            }
            catch (e:Exception){
                _userState.value = UserState.Error("Error: ${e.message}")
            }
        }
    }
}